@@ -1,7 +1,7 @@
 /*
  * parameter.h - parameter C API
  *
- *   Copyright (c) 2007-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2007-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -50,7 +50,9 @@
    storage.  It doesn't have extra features such as filter
    procedure or hooks.  It is useful for the parameter that needs
    to be accessed from C as well.
-   Scheme-level <parameter> object is built on top of this.
+
+   ScmParameter is Scheme's <parameter> object.  It inherits
+   primitive parameter, but adds some bells and whistles.
 */
 typedef struct ScmPrimitiveParameterRec ScmPrimitiveParameter;
 
@@ -59,6 +61,7 @@ SCM_CLASS_DECL(Scm_PrimitiveParameterClass);
 #define SCM_PRIMITIVE_PARAMETER(obj)   ((ScmPrimitiveParameter*)obj)
 #define SCM_PRIMITIVE_PARAMETER_P(obj) SCM_ISA(obj,SCM_CLASS_PRIMITIVE_PARAMETER)
 
+/* Flag value for Scm_MakePrimitiveParameter */
 enum {
     /* value may be a promise; dereference automaticlaly forces it */
     SCM_PARAMETER_LAZY = (1UL << 0)
@@ -68,9 +71,10 @@ SCM_EXTERN ScmPrimitiveParameter *Scm_MakePrimitiveParameter(ScmClass *klass,
                                                              ScmObj name,
                                                              ScmObj initval,
                                                              u_long flags);
-SCM_EXTERN ScmObj Scm_PrimitiveParameterRef(ScmVM *vm, 
+SCM_EXTERN ScmObj Scm_MakePrimitiveParameterSubr(ScmPrimitiveParameter *p);
+SCM_EXTERN ScmObj Scm_PrimitiveParameterRef(ScmVM *vm,
                                             const ScmPrimitiveParameter *p);
-SCM_EXTERN ScmObj Scm_PrimitiveParameterSet(ScmVM *vm, 
+SCM_EXTERN ScmObj Scm_PrimitiveParameterSet(ScmVM *vm,
                                             const ScmPrimitiveParameter *p,
                                             ScmObj val);
 
@@ -83,6 +87,7 @@ SCM_EXTERN ScmPrimitiveParameter *Scm_BindPrimitiveParameter(ScmModule *mod,
                                                              u_long flags);
 
 /* TRANSIENT - exposed only for the backward compatibility - will be gone by 1.0 */
+#if GAUCHE_API_VERSION < 98
 typedef struct ScmParameterLocRec {
     ScmPrimitiveParameter *p;
 } ScmParameterLoc;
@@ -94,11 +99,11 @@ SCM_EXTERN void Scm_DefinePrimitiveParameter(ScmModule *mod,
 SCM_EXTERN void   Scm_MakeParameterSlot(ScmVM *vm,
                                         ScmParameterLoc *location /*out*/);
 SCM_EXTERN void   Scm_InitParameterLoc(ScmVM *vm,
-                                       ScmParameterLoc *location, 
+                                       ScmParameterLoc *location,
                                        ScmObj initval);
 SCM_EXTERN ScmObj Scm_ParameterRef(ScmVM *vm, const ScmParameterLoc *location);
 SCM_EXTERN ScmObj Scm_ParameterSet(ScmVM *vm, const ScmParameterLoc *location,
                                    ScmObj value);
-
+#endif /*GAUCHE_API_VERSION < 98*/
 
 #endif /*GAUCHE_PARAMETER_H*/
