@@ -1,7 +1,7 @@
 ;;;
 ;;; binary.ftype - foreign types and foreign objects
 ;;;
-;;;   Copyright (c) 2011-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -31,14 +31,37 @@
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
 
-;; Experimental
+;; This module provides means to define and manipulate
+;; structured binary data.
+;;
+;; Foreign types and foreign objects
+;;
+;; A @emph{foreign type}, or @code{ftype}, refers to a way of representing
+;; external data used outside the Scheme world.  Ftypes are first-class
+;; objects in Gauche.  It can be used to extract or construct structured data
+;; from/in a plain bytevector (@code{u8vector}) and read from / write to
+;; ports.
+;;
+;; A @emph{foreign object} , or @code{fobject}, is a Scheme object
+;; that packages a chunk of binary data (@emph{storage}) with ftype.
+;; It behaves like a Scheme record---it can have slots and you can
+;; get and set values.
+;; However, it is possible to share storage among different Scheme objects.
+;; For example, you can read binary data (e.g. image file content)
+;; into an u8vector, then extract a part of it (e.g. image header)
+;; as an fobject.  When you do so, the extracted fobject shares its storage
+;; with the original u8vector; you can modify the header of the image
+;; content through the extracted fobject.  You may thing fobject as
+;; a typed reference to a certain memory.
 
 (define-module binary.ftype
   (use gauche.uvector)
   (use gauche.record)
-  (use srfi-1)
+  (use gauche.sequence)
+  (use scheme.list)
   (use srfi-13)
   (use binary.io)
+  (use util.match)
   (export
    ;; predefined primitive ftypes
    ftype:schar ftype:uchar  ftype:short ftype:ushort ftype:int ftype:uint
@@ -47,6 +70,8 @@
    ftype:int64 ftype:uint64
    ftype:float ftype:double
 
+   bitfield
+
    ;; type metainformation
    ftype ftype-name ftype-size ftype-alignment ftype-endian
    ftype-getter ftype-putter
@@ -64,8 +89,6 @@
    get-fobject put-fobject!
    read-fobject!/uv read-fobject write-fobject/uv write-fobject
    fobject-ref fobject-set! fobject-ref/uv fobject-set!/uv
-
-   describe-fstruct-type
    ))
 (select-module binary.ftype)
 
@@ -94,22 +117,16 @@
 (define-method write-object ((ftd ftype) port)
   (format port "#<ftype ~a>" (ftype-name ftd)))
 
-(define-record-type (ftype:struct ftype)
-  %make-ftype:struct #t
+(define-record-type (ftype:struct ftype) %make-ftype:struct #t
   slots                      ; an assoc list of name and ftype:slot
   )
 
-(define-record-type (ftype:array ftype)
-  %make-ftype:array #t
+(define-record-type (ftype:array ftype) %make-ftype:array #t
   (element-type ftype-element-type)     ; ftype
   (num-elements ftype-num-elements)     ; count
-  )
-
-(define-record-type (ftype:bitfield ftype)
-  %make-ftype:bitfield #t
-  type                                  ; ftype
-  bit-position                          ; start bit position
-  bit-width                             ; width
+  (element-alignment ftype-element-alignment) ; default to natural alignment
+                                              ; of element-type, can be
+                                              ; overridden.
   )
 
 ;; Auxiliary structure to keep each slot's info of ftype:struct.
@@ -120,6 +137,38 @@
   position          ; byte offset
   )
 
+;; Auxiliary structure to keep bitfield slot in ftype:struct.
+;; When fstruct is defined, bitfield slot needs to be deginated with
+;; bit-width and signed/unsigned.  The initializer of fstruct fills
+;; bit-position.
+(define-record-type (ftype:bitfield ftype) %make-ftype:bitfield #t
+  bit-position                          ; start bit position
+  bit-width                             ; width
+  )
+
+(define (bitfield width)
+  (make-ftype:bitfield #f #f width 0))
+
+(define (make-ftype:bitfield name signed? bitwidth bitpos)
+  (let* ([octets (div (+ bitwidth 7) 8)]
+         [size (ash 1 (integer-length (- octets 1)))]
+         [align size]
+         [geti (if signed? get-sint get-uint)]
+         [puti (if signed? put-sint! put-uint!)])
+    (%make-ftype:bitfield name size align #f
+                          (^[uv pos endian]
+                            (bit-field (geti uv pos endian)
+                                       bitpos
+                                       (+ bitpos bitwidth)))
+                          (^[uv pos val endian]
+                            (puti uv pos
+                                  (copy-bit-field (geti uv pos endian)
+                                                  val
+                                                  bitpos
+                                                  (+ bitpos bitwidth))))
+                          bitpos
+                          bitwidth)))
+
 ;;
 ;; Foreign object instance.
 ;;
@@ -242,7 +291,7 @@
 
 ;; slots :: ((name type) ...)
 (define (make-fstruct-type name slots endian alignment)
-  (receive (size slot-descriptors) (compute-fstruct-slots slots alignment)
+  (receive [slot-descriptors size] (compute-fstruct-slots slots alignment)
     (rec ftype
       (%make-ftype:struct name size
                           (or alignment (compute-fstruct-alignment slots))
@@ -262,21 +311,37 @@
 (define (compute-fstruct-alignment slots)
   (apply max (map (^p (ftype-alignment (cadr p))) slots)))
 
+;; Returns <<[ftype:slot] size>>
 (define (compute-fstruct-slots slots alignment)
-  (let loop ([slots slots]
-             [pos 0]
-             [descs '()])
-    (if (null? slots)
-      (values pos (reverse descs))
-      (let* ([name (caar slots)]
-             [type (cadar slots)]
-             [align (if alignment
-                      (min alignment (ftype-alignment type))
-                      (ftype-alignment type))]
-             [pos (%round pos align)])
-        (loop (cdr slots)
-              (+ pos (ftype-size type))
-              (acons name (make-ftype:slot name type pos) descs))))))
+  (define (do-slot slots ss pos)
+    (match slots
+      [() (values (reverse ss) pos)]
+      [([name type] . slots)
+       (let* ([align (if alignment
+                       (min alignment (ftype-alignment type))
+                       (ftype-alignment type))]
+              [pos (%round pos align)])
+         (if (ftype:bitfield? type)
+           (do-bitfield name type slots ss pos 0)
+           (do-slot slots
+                    `((,name . ,(make-ftype:slot name type pos)) ,@ss)
+                    (+ pos (ftype-size type)))))]))
+  (define (do-bitfield name type slots ss pos bitpos)
+    (let* ([w (ftype:bitfield-bit-width type)]
+           [s `(,name
+                . ,(make-ftype:slot name
+                                    (make-ftype:bitfield `(bits ,w ,bitpos)
+                                                         #f
+                                                         bitpos
+                                                         w)
+                                    pos))])
+      (match slots
+        [() (do-slot slots (cons s ss) (+ pos (div (+ bitpos 7) 8)))]
+        [([name type] . rest)
+         (if (ftype:bitfield? type)
+           (do-bitfield name type rest (cons s ss) pos (+ bitpos w))
+           (do-slot slots (cons s ss) (+ pos (div (+ bitpos 7) 8))))])))
+  (do-slot slots '() 0))
 
 ;;
 ;; Generic accessors.
@@ -331,20 +396,22 @@
 (define (fobject-ref/uv ftd slot uvector pos)
   (let1 s (%get-slot-desc ftd slot)
     ((ftype-getter (ftype:slot-type s))
-     uvector (+ pos (ftype:slot-position s)))))
+     uvector (+ pos (ftype:slot-position s))
+     #f)))                              ;endian
 
 (define (fobject-set!/uv ftd slot uvector pos val)
   (let1 s (%get-slot-desc ftd slot)
     ((ftype-putter (ftype:slot-type s))
-     uvector (+ pos (ftype:slot-position s)) val)))
+     uvector (+ pos (ftype:slot-position s)) val
+     #f)))                              ;endian
 
 (define (fobject-ref obj slot)
   (fobject-ref/uv (fobject-type obj) slot
                   (fobject-storage obj) (fobject-offset obj)))
 
 (define (fobject-set! obj slot val)
-  (fobject-ref/uv (fobject-type obj) slot
-                  (fobject-storage obj) val (fobject-offset obj)))
+  (fobject-set!/uv (fobject-type obj) slot
+                   (fobject-storage obj) (fobject-offset obj) val))
 
 ;; Initializes byte vector v, returns fobject of type ftype.
 ;; TODO: allow constructing fobjects other than fstructs.
@@ -420,29 +487,29 @@
 (define-macro (define-fstruct-type name ctor-name pred-name slots . options)
   (let* ([yname (unwrap-syntax name)]
          [maker (cond
-                 [(or (symbol? ctor-name) (identifier? ctor-name)) ctor-name]
-                 [(eq? ctor-name #t) (string->symbol #`"make-,yname")]
+                 [(identifier? ctor-name) ctor-name]
+                 [(eq? ctor-name #t) (string->symbol #"make-~yname")]
                  [(eq? ctor-name #f) #f]
                  ;; TODO: support inheritance like define-record-type
                  [else (error "invalid define-fstruct-type: ctor-name \
                                must be a symbol or a boolean, but got:"
                               ctor-name)])]
          [pred  (cond
-                 [(or (symbol? pred-name) (identifier? pred-name)) pred-name]
-                 [(eq? pred-name #t) (string->symbol #`",|yname|?")]
+                 [(identifier? pred-name) pred-name]
+                 [(eq? pred-name #t) (string->symbol #"~|yname|?")]
                  [(eq? pred-name #f) #f]
                  [else (error "invalid define-fstruct-type: pred-name \
                                must be a symbol or a boolean, but got:"
                               pred-name)])]
-         [initializer (string->symbol #`"init-,|yname|!")])
+         [initializer (string->symbol #"init-~|yname|!")])
     ;; Kludge to get hygienity
     (define (->id x) ((with-module gauche.internal make-identifier) x
                       (find-module 'binary.ftype) '()))
     (define (make-slot-procs sname type)
-      (let ([.ref  (string->symbol #`",|yname|-,|sname|")]
-            [.set! (string->symbol #`",|yname|-,|sname|-set!")]
-            [.get  (string->symbol #`"get-,|yname|-,|sname|")]
-            [.put! (string->symbol #`"put-,|yname|-,|sname|!")])
+      (let ([.ref  (string->symbol #"~|yname|-~|sname|")]
+            [.set! (string->symbol #"~|yname|-~|sname|-set!")]
+            [.get  (string->symbol #"get-~|yname|-~|sname|")]
+            [.put! (string->symbol #"put-~|yname|-~|sname|!")])
         `(begin
            (define (,.ref obj)
              (,(->id 'fobject-ref) obj ',sname))
@@ -476,11 +543,11 @@
 ;;
 ;; Debugging aid
 ;;
-(define (describe-fstruct-type fd)
+(define-method describe-slots ((fd ftype))
   (print "fstruct slots:")
+  (format #t " off slot      type\n")
   (dolist [slot (ftype:struct-slots fd)]
     (format #t " ~3d ~10a::~a~%"
             (ftype:slot-position (cdr slot))
             (car slot)
             (ftype-name (ftype:slot-type (cdr slot))))))
-
