@@ -1,7 +1,7 @@
 ;;;
 ;;; net - network interface
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,7 +34,7 @@
 #!no-fold-case
 
 (define-module gauche.net
-  (use srfi-1)
+  (use scheme.list)
   (use gauche.uvector)
   (use gauche.sequence)
   (use gauche.lazy)
