@@ -1,7 +1,7 @@
 ;;;
 ;;; auxiliary treemap utilities.  to be autoloaded.
 ;;;
-;;;   Copyright (c) 2007-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2007-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,10 +34,12 @@
 (define-module gauche.treeutil
   (export make-tree-map tree-map-empty?
           tree-map-min tree-map-max tree-map-pop-min! tree-map-pop-max!
-          tree-map-seek tree-map-fold tree-map-fold-right
+          tree-map-seek tree-map-seek-right
+          tree-map-fold tree-map-fold-right
           tree-map-map tree-map-for-each
           tree-map-keys tree-map-values
           tree-map->alist alist->tree-map
+          tree-map->generator/key-range
           tree-map-compare-as-sets
           tree-map-compare-as-sequences)
   )
@@ -82,7 +84,7 @@
 (define (tree-map-fold-right tm kons knil)
   (%tree-map-fold tm kons knil #t))
 
-(define (tree-map-seek tm pred succ fail)
+(define (%tree-map-seek tm pred fail succ backward)
   (assume-type tm <tree-map>)
   (let ((eof (cons #f #f))              ;marker
         (i ((with-module gauche.internal %tree-map-iter) tm)))
@@ -92,6 +94,12 @@
               [(pred k v) => (^r (succ r k v))]
               [else (loop)])))))
 
+(define (tree-map-seek tm pred fail succ)
+  (%tree-map-seek tm pred fail succ #f))
+
+(define (tree-map-seek-right tm pred fail succ)
+  (%tree-map-seek tm pred fail succ #t))
+
 (define (tree-map-map tm proc)
   (tree-map-fold-right tm (lambda (k v r) (cons (proc k v) r)) '()))
 
@@ -112,6 +120,46 @@
     (dolist (kv alist)
       (tree-map-put! tm (car kv) (cdr kv)))))
 
+;; Range generators.  Returns a pair of key/value while key is in the
+;; given range.
+(define (tree-map->generator/key-range tm :key ((:< <-key) #f)
+                                               ((:<= <=-key) #f)
+                                               ((:> >-key) #f)
+                                               ((:>= >=-key) #f)
+                                               (descending #f))
+  (define cmp (tree-map-comparator tm))
+  (define (call fn tm key)
+    (receive (k v) (fn tm key *uniq* *uniq*)
+      (if (eq? k *uniq*) #f (cons k v))))
+  (if descending
+    (let1 p (cond [<=-key (call tree-map-floor tm <=-key)]
+                  [<-key  (call tree-map-predecessor tm <-key)]
+                  [else   (tree-map-max tm)])
+      (^[]
+        (unless (and p (cond [>-key  (>? cmp (car p) >-key)]
+                             [>=-key (>=? cmp (car p) >=-key)]
+                             [else #t]))
+          (set! p #f))
+        (if p
+          (begin0 p
+            (set! p (call tree-map-predecessor tm (car p))))
+          (eof-object))))
+    ;; increasing
+    (let1 p (cond [>=-key (call tree-map-ceiling tm >=-key)]
+                  [>-key  (call tree-map-successor tm >-key)]
+                  [else   (tree-map-min tm)])
+      (^[]
+        (unless (and p (cond [<-key  (<? cmp (car p) <-key)]
+                             [<=-key (<=? cmp (car p) <=-key)]
+                             [else #t]))
+          (set! p #f))
+        (if p
+          (begin0 p
+            (set! p (call tree-map-successor tm (car p))))
+          (eof-object))))))
+
+(define *uniq* (cons #f #f))
+
 ;; Compare two tree-maps as sets.
 (define (tree-map-compare-as-sets tm1 tm2
                                   :optional (value=? equal?)
@@ -200,5 +248,4 @@
   (tree-map-compare-as-sequences a b))
 
 (define-method object-equal? ((a <tree-map>) (b <tree-map>))
-  (= (tree-map-compare-as-sequences a b)) 0)
-
+  (= (tree-map-compare-as-sequences a b) 0))
