@@ -38,6 +38,7 @@
 (select-module srfi-42)
 
 (autoload gauche.uvector uvector->list)
+(autoload data.range range? range-length range-ref)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -395,7 +396,7 @@
 
 ; (:while-1 cc test (:do ...))
 ;    modifies the fully decorated :do-generator such that it
-;    runs while test is a true value. 
+;    runs while test is a true value.
 ;       The original implementation just replaced ne1? by
 ;    (and ne1? test) as follows:
 ;
@@ -408,10 +409,10 @@
 ;    Unfortunately, this code is wrong because ne1? may depend
 ;    in the inner bindings introduced in ilet, but ne1? is evaluated
 ;    outside of the inner bindings. (Refer to the specification of
-;    :do to see the structure.) 
-;       The problem manifests itself (as sunnan@handgranat.org 
+;    :do to see the structure.)
+;       The problem manifests itself (as sunnan@handgranat.org
 ;    observed, 25-Apr-2005) when the :list-generator is modified:
-; 
+;
 ;      (do-ec (:while (:list x '(1 2)) (= x 1)) (display x)).
 ;
 ;    In order to generate proper code, we introduce temporary
@@ -435,9 +436,9 @@
 ;               /payload/
 ;               (if ne2?
 ;                   (loop ls ...) )))))
-; 
+;
 ; Bug #2:
-;    Unfortunately, the above expansion is still incorrect (as Jens-Axel 
+;    Unfortunately, the above expansion is still incorrect (as Jens-Axel
 ;    Soegaard pointed out, 4-Jun-2007) because ib-rhs are evaluated even
 ;    if ne1?-value is #f, indicating that the loop has ended.
 ;       The problem manifests itself in the following example:
@@ -578,7 +579,7 @@
           ((+ i 1)) ))
 
     ((_ *len *ref cc var (index i) arg1 arg2 arg ...)
-     (srfi-42-parallel cc 
+     (srfi-42-parallel cc
                        (srfi-42-*vector *len *ref cc var arg1 arg2 arg ...)
                        (srfi-42-integers i)) )
     ((_ *len *ref cc var arg1 arg2 arg ...)
@@ -619,6 +620,7 @@
     ((_ cc var)
      (srfi-42-do cc ((var 0)) #t ((+ var 1))) )))
 
+;; Extended to support srfi-196 (data.range) <range> object.
 
 (define-syntax srfi-42-range
   (syntax-rules (index)
@@ -627,7 +629,11 @@
     ((_ cc var (index i) arg1 arg ...)
      (srfi-42-parallel cc (srfi-42-range var arg1 arg ...) (srfi-42-integers i)) )
     ((_ cc var arg1)
-     (srfi-42-range cc var 0 arg1 1) )
+     ;; Avoid (range? arg1), for we don't want to load data.range until
+     ;; absolutely necessary.
+     (if (exact-integer? arg1)
+       (srfi-42-range cc var 0 arg1 1)
+       (srfi-42-*vector range-length range-ref cc var arg1)) )
     ((_ cc var arg1 arg2)
      (srfi-42-range cc var arg1 arg2 1) )
 
@@ -924,6 +930,10 @@
                (srfi-42-generator-proc (srfi-42-collection a1))]
               [(applicable? a1)
                (srfi-42-generator-proc (srfi-42-generator a1))]
+              ;; This would trigger autoloading data.range, so we don't
+              ;; support it for now.
+              ;; [(range? a1)
+              ;;  (srfi-42-generator-proc (srfi-42-range a1))]
               [else #f]))]
       [(2) (let ([a1 (car args)] [a2 (cadr args)])
              (cond
@@ -1239,4 +1249,3 @@
 
     ((every?-ec qualifier expression)
      (first-ec #t qualifier (if (not expression)) #f) )))
-
